@@ -1,7 +1,7 @@
 from django.contrib import admin
 
 #import local models
-from djhcup_staging.models import State, DataSource, FileType, File, Column, ImportQueue, ImportBatch, StagingTable, StagingBatch, StagingQueue
+from djhcup_staging.models import State, DataSource, FileType, File, Column, ImportQueue, ImportBatch, StagingTable, StagingBatch, StagingQueue, ReferenceColumn
 
 #Register models
 admin.site.register(State)
@@ -31,4 +31,5 @@ class StagingQueueInline(admin.StackedInline):
 class StagingBatchAdmin(admin.ModelAdmin):
     inlines = [StagingQueueInline]
 
-admin.site.register(StagingBatch, StagingBatchAdmin)
\ No newline at end of file
+admin.site.register(StagingBatch, StagingBatchAdmin)
+admin.site.register(ReferenceColumn)
\ No newline at end of file
