@@ -20,7 +20,6 @@ DB_DEF = settings.DATABASES[HACHOIR_DB_ENTRY]
 # start a logger
 default_logger = logging.getLogger('default')
 
-
 # Create your models here.
 class State(models.Model):
     abbreviation = models.CharField(max_length=2, blank=False,
@@ -98,7 +97,7 @@ class FileType(models.Model):
     hr_descriptor = property(_hr_descriptor)
     
     def __unicode__(self):
-        return 'FileType: %s %s' % (self.hr_descriptor, self.source)
+        return '<FileType: %s %s>' % (self.hr_descriptor, self.source)
 
 
 class File(models.Model):
@@ -158,7 +157,7 @@ class File(models.Model):
     
     
     def __unicode__(self):
-        return 'File: %s (%s)' % (self.filename, self.file_type)
+        return '<File: %s (%s)>' % (self.filename, self.file_type)
 
 
 class ImportBatch(models.Model):
@@ -782,24 +781,167 @@ class Column(models.Model):
     informat = models.CharField(max_length=100, blank=True, default=None, null=True,
                                 help_text='SAS load informat this column is based on, if applicable')
     STATUS_CHOICES = [
-        ('UNVERIFIED', 'Verification has not been attempted'),
+        ('NONE', 'Validation has not been attempted'),
         ('UNAVAILABLE_AUTO', 'HCUP summary stats automatically detected as unavailable'),
         ('UNAVAILABLE_MANUAL', 'HCUP summary stats manually detected as unavailable'),
-        ('VERIFIED_AUTO', 'Verified automatically by comparison with HCUP summary stats'),
-        ('VERIFIED_MANUAL', 'Verified manually by comparison with HCUP summary stats'),
-        ('VERIFIED_OVERRIDE', 'Marked as verified, overriding a known discrepancy with HCUP summary stats'),
-        ('FAILED_AUTO', 'Failed verification automatically by comparison with HCUP summary stats'),
-        ('FAILED_MANUAL', 'Failed verification manually by comparison with HCUP summary stats'),
-        ('FAILED_OVERRIDE', 'Marked as failed, overriding an automatic verification'),
+        ('VALID_AUTO', 'Validated automatically by comparison with HCUP summary stats'),
+        ('VALID_MANUAL', 'Validated manually'),
+        ('VALID_OVERRIDE', 'Marked as valid, overriding a known discrepancy with HCUP summary stats'),
+        ('FAILED_AUTO', 'Failed validation automatically by comparison with HCUP summary stats'),
+        ('FAILED_MANUAL', 'Failed validation manually by comparison with HCUP summary stats'),
+        ('FAILED_OVERRIDE', 'Marked as failed, overriding an automatic validation'),
     ]
-    integrity_status = models.CharField(max_length=100, choices=STATUS_CHOICES, blank=False, default='UNVERIFIED')
+    integrity_status = models.CharField(max_length=100, choices=STATUS_CHOICES, blank=False, default='NONE')
+    integrity_detail = models.TextField(blank=True, null=True)
     count_notnull = models.IntegerField(help_text='Count of non-null values.')
     count_null = models.IntegerField(help_text='Count of null values.')
-    min = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True,)
-    max = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True,)
-    stddev_pop = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True,)
-    stddev_samp = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True,)
-    mean = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True,)
+    min = models.DecimalField(max_digits=50, decimal_places=6,
+                              blank=True, default=None, null=True)
+    max = models.DecimalField(max_digits=50, decimal_places=6,
+                              blank=True, default=None, null=True)
+    stddev_pop = models.DecimalField(max_digits=50, decimal_places=6,
+                                     blank=True, default=None, null=True)
+    stddev_samp = models.DecimalField(max_digits=50, decimal_places=6,
+                                      blank=True, default=None, null=True)
+    mean = models.DecimalField(max_digits=50, decimal_places=6,
+                               blank=True, default=None, null=True)
+    
+    ref_col = models.ForeignKey('ReferenceColumn', blank=True, default=None,
+                                null=True)
     
     def __unicode__(self):
         return "<Column %s: %s.%s (%s non-null values)>" % (self.pk, self.stg.name, self.name, self.count_notnull)
+    
+    def match_ref_col(self, logger=None):
+        if logger is None:
+            logger = default_logger
+        
+        if self.ref_col == None:
+            # 
+            f = self.stg.imported_via.file
+            ds = f.file_type.source
+            st = f.state
+            yr = f.year
+            name = self.name
+            qs = ReferenceColumn.objects.filter(source=ds, state=st,
+                                                year=yr, name=name)
+            if len(qs) == 1:
+                self.ref_col = qs[0]
+                self.save()
+                logger.info("Successfully matched Column %s with ReferenceColumn %s" % (self.pk, self.ref_col.pk))
+                return True
+            elif len(qs) > 1:
+                logger.warning("Found %s ReferenceColumn entries matching Column %s; Column.integrity_status will be updated to UNAVAILABLE_AUTO. The entries were: %s." % (len(qs), self.pk, qs))
+                self.integrity_status = 'UNAVAILABLE_AUTO'
+                self.save()
+                return False
+            else:
+                logger.info("Found no ReferenceColumn entry matching Column %s; Column.integrity_status will be updated to UNAVAILABLE_AUTO.")
+                self.integrity_status = 'UNAVAILABLE_AUTO'
+                self.save()
+                return False
+        else:
+            logger.warning("match_ref_col() called on Column %s with ref_col already set to ReferenceColumn %s. match_ref_col() will not proceed." % (self.pk, self.ref_col.pk))
+            return False
+    
+    
+    def validate(self, tolerance=0.01):
+        """
+        Attempts to validate the integrity of data in this Column by
+        comparing summary stats with those in ReferenceColumn ref_col.
+        
+        Returns None if ref_col cannot be found, False if the
+        validation fails (observed values are outside of reference by
+        more than tolerance, where tolerance is a percentage), or True.
+        
+        Updates self.integrity_status based on results.
+        """
+        if self.ref_col == None:
+            # will attempt to match a reference column and re-do the
+            # validation.
+            match_attempt = self.match_ref_col()
+            if match_attempt == True:
+                return self.validate(self, tolerance=tolerance)
+            else:
+                return None
+        else:
+            all_summstat_attr = [
+                'count_notnull',
+                'count_null',
+                'min',
+                'max',
+                'stddev_samp', # HCUP uses this in their summary stats
+                'mean',
+            ]
+            
+            # comps are attribute names for comparison.
+            # we exclude any where Column.attr is None
+            comps = [i for i in all_summstat_attr
+                     if getattr(self, i) is not None]
+            
+            ref = self.ref_col
+            issues = []
+            results = []
+            for attr in comps:
+                # look through all the comparison attr
+                # and build a list based on comparisons:
+                # None for ref_col.attr == None
+                # True for self.attr between ref_col.attr*(1 plus-minus tolerance)
+                # else False
+                ref_val = getattr(ref, attr)
+                
+                if ref_val == None:
+                    results.append(None)
+                    issues.append("No reference value for %s" % attr)
+                else:
+                    obs_val = getattr(self, attr)
+                    ceiling = (1+tolerance)*ref_val
+                    floor = (1-tolerance)*ref_val
+                    if obs_val >= floor and obs_val <= ceiling:
+                        results.append(True)
+                    else:
+                        issue = "Observed value {obs_val} outside reference value {ref_val} * (1 plus-minus {tolerance})". \
+                            format(obs_val=obs_val, ref_val=ref_val, tolerance=tolerance)
+                        issues.append(issue)
+                        results.append(False)
+                
+                if self.integrity_detail == None:
+                    self.integrity_detail = '\n'.join(issues)
+                else:
+                    self.integrity_detail += '\n'.join(issues)
+                
+                if False in results:
+                    self.integrity_status = 'FAILED_AUTO'
+                    self.save()
+                    return False
+                elif True in results:
+                    self.integrity_status = 'VERIFIED_AUTO'
+                    self.save()
+                    return True
+                else:
+                    # no reference values were found whatsoever for
+                    # those on the Column itself.
+                    self.integrity_status = 'UNAVAILABLE_AUTO'
+                    self.save()
+                    return None
+
+
+class ReferenceColumn(models.Model):
+    """Records details on reference values scraped from HCUP documentation.
+    
+    Uses source, state, and year to be later matched with a Column record. 
+    """
+    source = models.ForeignKey('DataSource')
+    state = models.ForeignKey('State')
+    year = models.IntegerField(blank=False, null=False, db_index=True)
+    name = models.CharField(max_length=200, blank=False, db_index=True)
+    
+    count_notnull = models.IntegerField(help_text='Count of non-null values.')
+    count_null = models.IntegerField(help_text='Count of null values.')
+    min = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True)
+    max = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True)
+    stddev_samp = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True)
+    mean = models.DecimalField(max_digits=50, decimal_places=6, blank=True, default=None, null=True)
+    
+    def __unicode__(self):
+        return "<ReferenceColumn %s: %s %s %s %s>" % (self.pk, self.source, self.state.abbreviation, self.year, self.name)
