@@ -205,7 +205,7 @@ def import_dispatch(self, importqueue_pk, insert_size=5, slice_size=1000):
     
     # FROM HERE BELOW, SPLIT OFF PSYCOPG2 VS NOT
     if ('psycopg2' in DB_BACKEND and (
-            ((dsabbr.upper() in ['CORE', 'BASE', 'CHGS', 'DTE', 'AHAL']) or
+            ((dsabbr.upper() in ['CORE', 'BASE', 'CHGS', 'DTE', 'AHAL', 'DX_PR_GRPS']) or
                 (dsabbr.upper() in ['CHGS'] and meta_is_long(load.get_meta()))
                 )
             )
@@ -343,6 +343,7 @@ def pg_import_file(self, importqueue_pk, drop_rawload=True):
 
         # v is a series of values in row with dict-style access by label
         for k, v in m_aug.iterrows():
+            load.log("Creating Column record for %s for StagingTable %s" % (v['field'], imported.pk))
             
             # create Column object with preliminary values
             col = Column(
@@ -352,7 +353,7 @@ def pg_import_file(self, importqueue_pk, drop_rawload=True):
                 col_type=v['data_type'].upper(),
                 col_scale=v['scale'],
                 col_precision=v['length'],
-                informat=v['informat'],
+                informat=v['informat']
                 )
             
             if v['numeric_or_int']:
@@ -397,7 +398,10 @@ def pg_import_file(self, importqueue_pk, drop_rawload=True):
                 col.count_null = result[1]
             
             col.save()
-            load.log('Recorded Column %s for StagingTable %s' % (col.pk, imported.pk))
+            load.log('Recorded Column %s' % (col.pk))
+            load.log('Attempting to validate Column %s' % (col.pk))
+            v_result = col.validate()
+            load.log('Validation status for Column %s is %s' % (col.pk, col.integrity_status))
         
         """
         else:
